//precision mediump float;
//uniform vec2 iResolution;
//uniform float iTime;

#define M_PI 3.1415926535897932384626433832795

float plot(vec2 st, float pct){
  return  smoothstep( pct-0.01, pct, st.y) -
          smoothstep( pct, pct+0.01, st.y);
}

float plot_y(vec2 st, float pct, float w){
  return  smoothstep( pct-w, pct, st.y) -
          smoothstep( pct, pct+w, st.y);
}

float plot_x(vec2 st, float pct, float w){
  return  smoothstep( pct-w, pct, st.x) -
          smoothstep( pct, pct+w, st.x);
}

float spot(vec2 p, float r) {
  float d = length(p);
  return smoothstep(0.,d,r);
}


vec3 bgcolor(vec2 uv) {
  float m1 = 0.;
  m1 = clamp(.0, 0.5, uv.y*uv.y );
  return vec3(0.,0., m1);
}

vec3 sun(vec2 uv) {
  float t = fract(iTime*0.03);

  vec2 p = uv;
  p.y -= (t*0.4 - .0625);
  vec3 basecol = vec3(0.0,0.0,0.05) ;
  basecol.r += smoothstep( 0.28, 1.5, t*3.4)*0.80 ;
  basecol.g += smoothstep( 0.32, 1.5, t*3.2)*0.75 ;
  basecol.b += smoothstep( 0.38, 1.5, t*3.2)*0.80 ;
  vec3 col=basecol;
  vec2 m = vec2(0.01,0.01);
  return col*smoothstep(0.1, -0.1 , length(p) )*smoothstep(-0.03,0.01,uv.y);
}

vec3 sky(vec2 uv) {
  float t = fract(iTime*0.03);
  //t *=0.90;
  vec3 basecol = vec3(0.0,0.0,0.05) ;
  basecol.r += smoothstep( 0.30, 1.2, t*2.8 )*0.79 ;
  basecol.g += smoothstep( 0.34, 1.2, t*2.6 )*0.80 ;
  basecol.b += smoothstep( 0.40, 1.2, t*2.5 )*0.95 ;
  vec3 col=basecol;
  return col*smoothstep(-0.2, 0.1 ,uv.y )*smoothstep(-0.4, 0.005,uv.y);
}

vec3 ocean(vec2 uv, vec3 basecol) {
  vec3 col=basecol;
  //*smoothstep(-1.,0.,uv.y);
  return col*step(uv.y, 0.) ;
}

void mainImage( out vec4 fragColor, in vec2 fragCoord ) {
  vec2 uv = fragCoord.xy / iResolution.xy ;
  uv -= .5;
  float asp = iResolution.x/iResolution.y;
  uv.x *= asp;

  vec3 col =  vec3(0.);
  col += bgcolor(uv);
  vec3 cl = vec3(0,0,0.18);
  col = ocean(uv, cl) ;
  col += sky(uv) ;
  col += sun(uv) ;

  fragColor = vec4(col, 1.) ;
}
 }
    e /= 6.;
    return e;
}

vec3 waterNormals(vec2 uv){
    uv.x *= .25;
    float eps = 0.008;    
    vec3 n=vec3( waterHeight(uv) - waterHeight(uv+vec2(eps,0.)),
                 1.,
                 waterHeight(uv) - waterHeight(uv+vec2(0.,eps)));
   return normalize(n);
}	


vec3 drawSky( vec2 uv, vec2 uvInit){ 
        
	float clouds = noiseLayer(uv);
    
    // clouds normals
    float eps = 0.1;
    vec3 n = vec3(	clouds - noiseLayer(uv+vec2(eps,0.)),
            		-.3,
             		clouds - noiseLayer(uv+vec2(0.,eps)));
    n = normalize(n);
    
    // fake lighting
    float l = dot(n, normalize(vec3(uv.x,-.2,uv.y+.5)));
    l = clamp(l,0.,1.);
    
    // clouds color	(color gradient from light)
    vec3 cloudColor = mix(baseSkyColor, darkColor, length(uvInit)*1.7);
    cloudColor = mix( cloudColor,sunColor, l );
    
    // sky color (color gradient on Y)
    vec3 skyColor = mix(lightColor , baseSkyColor, clamp(uvInit.y*2.,0.,1.) );
    skyColor = mix ( skyColor, darkColor, clamp(uvInit.y*3.-.8,0.,1.) );
    skyColor = mix ( skyColor, sunColor, clamp(-uvInit.y*10.+1.1,0.,1.) );
    
	// draw sun
    if(length(uvInit-vec2(0.,.04) )<.03){
     	skyColor += vec3(2.,1.,.8);
    }
       
   	// mix clouds and sky
    float cloudMix = clamp(0.,1.,clouds*4.-8.);
    vec3 color = mix( cloudColor, skyColor, clamp(cloudMix,0.,1.) );
    
    // draw islands on horizon
    uvInit.y = abs(uvInit.y);
    float islandHeight = texture(iChannel1, uvInit.xx/2.+.67).r/15. - uvInit.y + .978;
    islandHeight += texture(iChannel1, uvInit.xx*2.).r/60.;
    islandHeight = clamp(floor(islandHeight),0.,1.);    
    vec3 landColor = mix(baseSkyColor, darkColor, length(uvInit)*1.5);
    color = mix(color, landColor, islandHeight);

    return color;
}

void main()
{
    // center uv around horizon and manage ratio
	vec2 uvInit = gl_FragCoord.xy / iResolution.xy;
    uvInit.x -= .5;
    uvInit.x *= iResolution.x/iResolution.y;	
    uvInit.y -= 0.35;
    
    // perspective deform 
    vec2 uv = uvInit;
    uv.y -=.01;
	uv.y = abs(uv.y);
    uv.y = log(uv.y)/2.;
    uv.x *= 1.-uv.y;
    uv *= .2;
    
    vec3 col = vec3(1.,1.,1.);
    
    // draw water
    if(uvInit.y < 0.){       
       
        vec3 n = waterNormals(uv);
        
        // draw reflection of sky into water
        vec3 waterReflections = drawSky(uv+n.xz, uvInit+n.xz);

        // mask for fore-ground green light effect in water
        float transparency = dot(n, vec3(0.,.2,1.5));        
        transparency -= length ( (uvInit - vec2(0.,-.35)) * vec2(.2,1.) );
		transparency = (transparency*12.+1.5);
        
        // add foreground water effect
        waterReflections = mix( waterReflections, seaColor, clamp(transparency,0.,1.) );
        waterReflections = mix( waterReflections, seaLight, max(0.,transparency-1.5) );

       	col = waterReflections;
        
        // darken sea near horizon
       	col = mix(col, col*vec3(.6,.8,1.), -uv.y);
        
        //sun specular
        col += max(0.,.02-abs(uv.x+n.x))* 8000. * vec3(1.,.7,.3) * -uv.y * max(0.,-n.z);
        
    }else{      
        
        // sky
        col = drawSky(uv, uvInit);
    }
    
    // sun flare & vignette
    col += vec3(1.,.8,.6) * (0.55-length(uvInit)) ;
    
    // "exposure" adjust
    col *= .75;
    col = gamma(col,1.3);
    
    fragColor = vec4(col,1.);
}

//--------------------------------------------------------------------
//There is no salvation in becoming adapted to a world which is crazy.

    fragColor = vec4(col,1.);
}

//--------------------------------------------------------------------
//There is no salvation in becoming adapted to a world which is crazy.
