#version 100

attribute vec3 inPosition;
attribute vec3 inNormal;
attribute vec2 inTexCoord;

uniform mat4 matWorld;
uniform mat4 matWVP;
uniform vec4 fogParam[3];
uniform vec3 viewPos;
uniform vec4 alphaColor;

varying vec4 oFogColor;
varying vec4 color;
varying vec2 texCoord;
varying vec3 normal;
varying vec3 viewDir;

invariant gl_Position;

float ComputeFog(vec3 camToWorldPos, vec3 param)
{
	float fdist = max(length(camToWorldPos) - param.x, 0.0);
	float density = clamp(clamp(fdist/(param.y-param.x), 0.0, 1.0) * param.z, 0.0, 1.0);
	return 1.0 - density;
}

void main(void)
{
	vec4 vWorldPos = matWorld * vec4(inPosition, 1.0);
	gl_Position = matWVP * vec4(inPosition, 1.0);

	texCoord = inTexCoord;
	normal = normalize(mat3(matWorld) * inNormal);
	viewDir = viewPos - vWorldPos.xyz;

	color = vec4(1.0, 1.0, 1.0, 1.0);
	color.a = alphaColor.a;

	oFogColor = vec4(fogParam[1].rgb, ComputeFog(vWorldPos.xyz - fogParam[2].xyz, fogParam[0].xyz));
}
